/*
    This program should randomly open files data0.txt through
    data9.txt and copy their contents to files sequentially
    named output1,txt, output2.txt, output3.txt, etc. In addition,
    this program should do a sequence of randomly parameterized
    compute bound tasks.

    This program uses the Timing.h functions to keep fairly
    accurate timing information.
*/
#include "constants.h"
#include "printError.h"
#include "Timing.h"
#include <stdio.h>
#include <math.h>
#include <windows.h>

int main(int argc, char *argv[])
{
   char  inFileName[LENGTH_OF_FILE_NAMES];
   char outFileName[LENGTH_OF_FILE_NAMES];
   int n;

   TimingObject threadStartTime,  threadEndTime,  threadElapsedTime,
                processStartTime, processEndTime, processElapsedTime,
                taskStartTime,    taskEndTime,    taskElapsedTime;

   /* keep track of when we start working */
   getProcessTime(&processStartTime);
   getThreadTime(&threadStartTime);

   /* this loop does all of the I/O tasks */
   for (n = 0; n < NUMBER_OF_IO_TASKS; n++)
   {
		/* keep track of when this task begins */
		getThreadTime(&taskStartTime);

      // Create an input file name that randomly chooses one
      // of the ten input files, data0.txt through data9.txt.

      // Create an output file name that sequentially numbers the
      // output files as output?.txt where ? is an integer from 1
      // to NUMBER_OF_IO_TASKS.

      // Open the input file, using the Win32 function CreateFile(),
      // for shared reading (be sure to do error checking).

      // Open the output file, using the Win32 function CreateFile(),
      // for writing (be sure to do error checking).

      // Copy the input file to the output file (and error check all
      // API calls). Be sure to do this in a way that does not use
      // up too much CPU time (so do not copy one file to the other
      // byte by byte!).

      // Note: You may want to log some (but not too much) information
      // to the console to help you see what is going on.

      /* determine when this task finishes */
      getThreadTime(&taskEndTime);
      computeElapsedTime(&taskStartTime, &taskEndTime, &taskElapsedTime);
      printTimingObject(&taskElapsedTime);
   }

   /* this loop does all of the compute tasks */
   for (n = 0; n < NUMBER_OF_CALC_TASKS; n++)
   {
      int multiplier;

      /* keep track of when this task begins */
      getThreadTime(&taskStartTime);

      // Randomly choose an integer "multiplier".

      // Create a loop that does a lot of calculations. Use the
      // multiplier and the COMPUTE_COUNTER constant to "parameterize"
      // how many calculations this loop does. The multiplier should
      // be a positive integer, so that, for example, multiplier=2 will
      // cause this loop to do twice as many calculations as multiplier=1.
      // When the multiplier is 1, this loop should take about as much
      // time as the I/O loop takes to copy the smallest of the
      // ten data files.

      // Note: You may want to log some (but not too much) information
      // to the console to help you see what is going on.

      /* determine when this task finishes */
      getThreadTime(&taskEndTime);
      computeElapsedTime(&taskStartTime, &taskEndTime, &taskElapsedTime);
      printTimingObject(&taskElapsedTime);
   }


   /* determine when we finish working */
   getThreadTime(&threadEndTime);    // There shouldn't be much difference
   getProcessTime(&processEndTime);  // between these two timings.

   computeElapsedTime(&threadStartTime, &threadEndTime, &threadElapsedTime);
   printTimingObject(&threadElapsedTime);

   computeElapsedTime(&processStartTime, &processEndTime, &processElapsedTime);
   printTimingObject(&processElapsedTime);

   return 0;
}
