/*
    This program does the work tasks using threads,
    one thread for each input file to copy and one
    thread for each (parameterized) calculation.

    The main() function creates the worker threads and
    passes to them the parameters that they need.

    The IOthread() and CALCthread() thread functions do the work.
*/
#include "constants.h"
#include "printError.h"
#include "Timing.h"
#include <stdio.h>
#include <math.h>
#include <windows.h>

/* Data structure needed for passing parameters to the I/O threads */
typedef struct
{
   char inFileName[LENGTH_OF_FILE_NAMES];
   char outFileName[LENGTH_OF_FILE_NAMES];
}ThreadStuff_t;


int main(int argc, char *argv[])
{
   /* thread function prototypes */
   DWORD WINAPI IOthread(LPVOID);
   DWORD WINAPI CALCthread(LPVOID);

   #define NUMBER_OF_IO_THREADS    NUMBER_OF_IO_TASKS
   #define NUMBER_OF_CALC_THREADS  NUMBER_OF_CALC_TASKS
   #define NUMBER_OF_THREADS NUMBER_OF_IO_THREADS + NUMBER_OF_CALC_THREADS

   HANDLE lpHandles[NUMBER_OF_THREADS];
   DWORD threadID;

   int threadCount, n;

   /* array to hold I/O thread parameters */
   ThreadStuff_t threadParams[NUMBER_OF_IO_THREADS];

   /* array to hold Calc thread parameters */
   int multiplier[NUMBER_OF_CALC_THREADS];

   TimingObject startTime,  endTime,  elapsedTime;

   threadCount = 0;

   /* keep track of when we start working */
   getProcessTime(&startTime);

   /* this loop creates all of the I/O worker threads */
   for (n = 0; n < NUMBER_OF_IO_THREADS; n++)
   {
      // Randomly choose an input file name.
      // Create a properly sequenced output file name.
      // Put this data in an instance of the ThreadStuff_t
      // data structure.

      // Use CreateThread to create a worker thread. Pass
      // a pointer to the thread parameter data structure to
      // CreateThread. Keep track of the handle returned by
      // CreateThread; it is needed to determine when the thread
      // terminates. (Note: Don't confuse the thread handle
      // returned by CreateThread with the thread ID also returned
      // by CreateThread. The thread ID is interesting, but not
      // necessary for this program.)

      // Note: You may want to log some (but not too much) information
      // to the console to help you see what is going on.
   }

   /* this loop creates all of the compute worker threads */
   for (n = 0; n < NUMBER_OF_CALC_THREADS; n++)
   {
      // Randomly choose an integer "multiplier".

      // Use CreateThread to create a worker thread. Pass to
      // CreateThread a pointer to the multiplier parameter.
      // Keep track of the handle returned by CreateThread;
      // it is needed to determine when the thread terminates.

      // Note: You may want to log some (but not too much) information
      // to the console to help you see what is going on.
   }

   /* wait for all worker threads to end */
   printf("  waiting for all threads to complete...");
   // NOTE: WaitForMultipleObjects() can only wait on a maximum of 64 objects.
   // To wait on more than 64 threads you need to call WaitForMultipleObjects()
   // several times. If 64 < threadCount <= 128, call WaitForMultipleObjects()
   // first with 64 as the first paramenter and lpHandles as the second parameter,
   // then call WaitForMultipleObjects() a second time with threadCount-64 as the
   // first parameter and with lpHandles+64 as the second parameter. Similarly if
   // threadCount > 128. In other words, you need to "walk" through the lpHandles[]
   // array in steps of 64. Also, the calls to WaitForMultipleObjects() should be
   // error checked.
   //WaitForMultipleObjects(threadCount, lpHandles, TRUE, INFINITE);
   printf("\nAll threads completed.\n");

   /* determine when we finish working */
   getProcessTime(&endTime);
   computeElapsedTime(&startTime, &endTime, &elapsedTime);
   printTimingObject(&elapsedTime);

   for (n = 0; n < threadCount; n++)
   {
      printThreadTimes(lpHandles[n]);
   }

   return 0;
}



DWORD WINAPI IOthread(LPVOID threadParam)
{
   /* This is how we get access to the thread parameters */
   ThreadStuff_t* threadParams_p = (ThreadStuff_t*)threadParam;

   // Get the input and output file names from the thread parameter
   // data structure.

   // Use CreateFile to open the input file for shared reading and
   // use CreateFile again to open the output file for writing.
   // (be sure to error check these function calls)

   // Copy the input file to the output file (and error check all
   // API calls). Be sure to do this in a way that does not use
   // up too much CPU time.

   return 0;
}//IOthread



DWORD WINAPI CALCthread(LPVOID threadParam)
{
   // Create a loop that does a lot of calculations. Use the
   // multiplier thread parameter and the COMPUTE_COUNTER constant
   // to "parameterize" how many calculations this loop does. The
   // multiplier should be a positive integer, so that, for example,
   // multiplier=2 will cause this loop to do twice as many
   // calculations as multiplier=1. When the multiplier is 1, this
   // thread should take about as much time as an I/O thread copying
   // the smallest of the ten data files.

   return 0;
}//CALCthread
